<?php

require_once DIR_SYSTEM . 'library/VerifoneEcomApi/autoload.php';

use VerifoneEcomApi\ApiWrapper\Authentication\BasicAuth;
use VerifoneEcomApi\ApiWrapper\Http\ApiClient\Client;
use VerifoneEcomApi\ApiWrapper\Http\SimpleCurl;
use VerifoneEcomApi\ApiWrapper\Regions\Emea;
use VerifoneEcomApi\ApiWrapper\Regions\Na;
use VerifoneEcomApi\ApiWrapper\Regions\NzAu;
use VerifoneEcomApi\ApiWrapper\Settings;

class ModelExtensionPaymentVerifoneHosted extends Model
{
    const OPENCART_REFUND = 11;
    private $mapper;
    private $simpleCurl;
    private $client;
    private $auth;
    private $settings;

    /**
     * install the module & creates the table
     */
    public function install()
    {
        $this->db->query("
            CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "verifone_hosted` (
            `id` INT(11) NOT NULL AUTO_INCREMENT ,
            `order_id` INT(11) NOT NULL ,
            `amount` DECIMAL(10,2) NOT NULL ,
            `currency` VARCHAR(3) NULL ,
            `transaction_id` VARCHAR(50) NOT NULL ,
            `verifone_order_status` VARCHAR(50) NOT NULL ,
            `created_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
            PRIMARY KEY (`id`)
            ) ENGINE = InnoDB;");

        $this->db->query("
            CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "verifone_hosted_refunds` (
            `id` INT(11) NOT NULL AUTO_INCREMENT ,
            `order_id` INT(11) NOT NULL ,
            `amount_refunded` DECIMAL(10,2) NOT NULL ,
            `verifone_refund_status` VARCHAR(50) NOT NULL ,
            `comment` TEXT NULL ,
            `response` TEXT NULL ,
            `transaction_id` TEXT NULL ,
            `created_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
            PRIMARY KEY (`id`)
            ) ENGINE = InnoDB;");
    }

    /**
     * uninstall and drop the table
     */
    public function uninstall()
    {
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "verifone_hosted`;");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "verifone_hosted_refunds`;");
    }

    /**
     *
     */
    public function setUpVerifoneSettings()
    {
        $this->settings = new Settings();
        $this->settings->setTest('yes' == $this->config->get('payment_verifone_hosted_test'))
            ->setRegion($this->config->get('payment_verifone_hosted_region'))
            ->setApiUserId($this->config->get('payment_verifone_hosted_prod_api_user'))
            ->setApiKey($this->config->get('payment_verifone_hosted_prod_api_key'))
            ->setTestApiUserId($this->config->get('payment_verifone_hosted_test_api_user'))
            ->setTestApiKey($this->config->get('payment_verifone_hosted_test_api_key'))
            ->set3dsEnable('yes' == $this->config->get('payment_verifone_hosted_3ds'))
            ->set3dsContractId($this->config->get('payment_verifone_hosted_prod_3ds_contract'))
            ->setPaymentContractId($this->config->get('payment_verifone_hosted_prod_contract'))
            ->setEntityId($this->config->get('payment_verifone_hosted_prod_entity'))
            ->setTestEntityId($this->config->get('payment_verifone_hosted_test_entity'))
            ->setTestPaymentContractId($this->config->get('payment_verifone_hosted_test_contract'))
            ->setTest3dsContractId($this->config->get('payment_verifone_hosted_test_3ds_contract'))
            ->setCurlVerifySsl('1' == $this->config->get('payment_verifone_hosted_curl'))
            ->setCvvOnPayment('1' == $this->config->get('payment_verifone_hosted_cvv'));
    }

    /**
     *
     */
    public function getRegionMapper()
    {
        // mini factory :D
        switch ($this->config->get('payment_verifone_hosted_region')) {
            case 'NZAU':
                $this->mapper = new Nzau($this->settings);
                break;
            case 'NA':
                $this->mapper = new Na($this->settings);
                break;
            case 'EMEA':
                $this->mapper = new Emea($this->settings);
                break;
            default:
                // will happen only on the first installation
                $this->mapper = new Nzau($this->settings);
        }
    }

    /**
     * @param $order_id
     * @return mixed
     */
    public function getTransactionByOrderId($order_id)
    {
        $query = $this->db->query("SELECT * FROM `" . DB_PREFIX . "verifone_hosted` WHERE `order_id` = '" . (int)$order_id . "' ORDER BY `created_at` DESC LIMIT 1");

        if ($query->num_rows) {
            $transaction = $query->row;
            $q = $this->db->query("SELECT * FROM `" . DB_PREFIX . "verifone_hosted_refunds` WHERE `order_id` = '" . $order_id . "'");
            $transaction['refunds'] = ($q->num_rows) ? $q->rows : [];

            return $transaction;
        }

        return null;
    }

    public function createRefund($order, $transaction, $amount, $comment)
    {
        $data = [
            'amount' => (int)($amount * 100),
            'reason' => (string)$comment,
            'id' => (string)$transaction['transaction_id'],
            'reference_id' => (string)$order,
            'created_date_time' => date('Y-m-d\TH:i:s\Z', strtotime($transaction['created_at']))
        ];
        return $data;
    }


    /**
     * @param        $order_id
     * @param        $amount
     * @param string $comment
     * @return array
     * @throws \Exception
     */
    public function refund($order_id, $amount, $comment = '')
    {
        $this->load->model('sale/order');
        $order = $this->model_sale_order->getOrder($order_id);
        $transaction = $this->getTransactionByOrderId($order_id);
        $available_amount = $transaction['amount'];
        $amount = number_format((float)$amount, 2, '.', '');
        if ($transaction['refunds']) {
            foreach ($transaction['refunds'] as $refund) {
                if ($refund['verifone_refund_status'] === 'REFUNDED') {
                    $available_amount = $available_amount - $refund['amount_refunded'];
                }
            }
        }
        $available_amount = round($available_amount, 2);
        if ($amount > $available_amount) {
            return [
                'success' => false,
                'message' => 'Amount that can be refunded must be between 0.1 '
                    . $order['currency_code'] . ' and ' . $available_amount . ' ' . $order['currency_code'] . '!'
            ];
        }

        $this->simpleCurl = new SimpleCurl();
        $this->setUpVerifoneSettings();
        $this->getRegionMapper();
        $this->auth = new BasicAuth($this->settings);
        $this->client = new Client(
            $this->mapper,
            $this->settings,
            $this->auth,
            $this->simpleCurl
        );

        $params = $this->createRefund($order_id, $transaction, $amount, $comment);

        try {
            $result = $this->client->postRefund($params, $transaction["transaction_id"]);
        } catch (Exception $e) {

            return [
                'success' => false,
                'message' => $e->getMessage(),
                'full_refund' => false,
                'row' => '<tr><td colspan="5">Something went wrong. The request was not processed!</td></tr>'
            ];
        }

        //REFUNDED - ok , other statuses are not ok
        if (!isset($result['id']) || !isset($result['status']) || 'REFUNDED' !== $result['status']) { // with error
            $success = false;
            $response = 'Something went wrong, please try again later or contact us';
            return [
                'success' => $success,
                'message' => $response,
                'full_refund' => false,
                'row' => '<tr><td colspan="5">Something went wrong. The request was not processed!</td></tr>'
            ];
        } else { // success response, but not necessary a success refund
            $status = $result['status'];
            $success = true;
            $response = 'Successfully refunded';
        }
        $full_refund = false;
        // if is a full refund we update the order status
        if ($amount == $available_amount) {
            $this->setOrderAsRefunded($order_id);
            $full_refund = true;
        }

        return [
            'success' => $success,
            'message' => $response,
            'full_refund' => $full_refund,
            'row' => $this->createRefundTransaction($status, $response, $order_id, $amount, $comment, $result['id'])
        ];
    }

    /**
     * @param $status
     * @param $response
     * @param $order_id
     * @param $amount
     * @param $comment
     * @return mixed
     */
    private function createRefundTransaction($status, $response, $order_id, $amount, $comment, $transaction_id)
    {

        $insert = $this->db->query("
                INSERT INTO `" . DB_PREFIX . "verifone_hosted_refunds` SET
                `order_id` = '" . $order_id . "',
                `amount_refunded` = '" . $amount . "',
                `verifone_refund_status` = '" . $this->db->escape($status) . "',
                `comment` = '" . $this->db->escape($comment) . "',
                `response` = '" . $this->db->escape($response) . "',
                `transaction_id` = '" . $this->db->escape($transaction_id) . "'
                ");
        if (!$insert) {
            return '<tr><td colspan="5">Something went wrong. The request was not processed!</td></tr>';
        }
        $class = $status === 'REFUNDED' ? 'text-success' : 'text-warning';
        $formatted_status = str_replace('_', ' ', $status);

        return '<tr>
                    <td>' . $amount . '</td>
                    <td><strong class="' . $class . '">' . $formatted_status . '</strong></td>
                    <td>' . $response . '</td>
                    <td>' . $comment . '</td>
                    <td>' . date('Y-m-d H:i:s', time()) . '</td>
               </tr>';
    }

    /**
     * @param $order_id
     */
    private function setOrderAsRefunded($order_id)
    {

        $this->db->query("UPDATE `" . DB_PREFIX . "order` SET 
                `order_status_id` =  '" . (int)$this->config->get('payment_verifone_hosted_refunded_status_id') . "'
                WHERE `order_id` = '" . $order_id . "'");

        $this->db->query("INSERT INTO `" . DB_PREFIX . "order_history` SET 
                `order_id` =  '" . $order_id . "',
                `order_status_id` =  '" . (int)$this->config->get('payment_verifone_hosted_refunded_status_id') . "',
                `date_added` =  '" . date('Y-m-d H:i:s', time()) . "',
            `comment` =  'Full amount refunded'");

    }
}
