<?php

require_once DIR_SYSTEM . 'library/VerifoneEcomApi/autoload.php';

class ControllerExtensionModuleVerifoneHosted extends Controller {

    const VERIFONE_ORDER_TRANSACTION_ID = '__verifone_order_transaction_id';
    const VERIFONE_ORDER_REFUND_ID = '__verifone_order_refund_id';
    const VERIFONE_TRANSACTION_AUTHORISED = 'AUTHORISED';
    const VERIFONE_TRANSACTION_SALE_AUTHORISED = 'SALE AUTHORISED';
    const VERIFONE_TRANSACTION_SALE_SETTLED = 'SALE SETTLED';
    const VERIFONE_TRANSACTION_AUTHORISATION_AUTHORISED = 'AUTHORISATION AUTHORISED';
    const VERIFONE_TRANSACTION_REFUND_AUTHORISED = 'REFUND AUTHORISED';
    const VERIFONE_TRANSACTION_REFUND_REFUNDED = 'REFUND REFUNDED';
    const VERIFONE_TRANSACTION_REFUND_SETTLED = 'REFUND SETTLED';
    const VERIFONE_TRANSACTION_REFUNDED = 'REFUNDED';
    const VERIFONE_TRANSACTION_PENDING = 'PENDING';
    const VERIFONE_WEBHOOK_EVENT_TYPE_AUTHORISATION_APPROVED = 'TxnAuthorisationApproved';
    const VERIFONE_WEBHOOK_EVENT_TYPE_AUTHORISATION_DECLINED = 'TxnAuthorisationDeclined';
    const VERIFONE_WEBHOOK_EVENT_TYPE_CAPTURE_APPROVED = 'TxnCaptureApproved';
    const VERIFONE_WEBHOOK_EVENT_TYPE_CAPTURE_DECLINED = 'TxnCaptureDeclined';
    const VERIFONE_WEBHOOK_EVENT_TYPE_REFUND_APPROVED = 'TxnRefundApproved';
    const VERIFONE_WEBHOOK_EVENT_TYPE_REFUND_DECLINED = 'TxnRefundDeclined';
    const VERIFONE_WEBHOOK_EVENT_TYPE_SALE_APPROVED = 'TxnSaleApproved';
    const VERIFONE_WEBHOOK_EVENT_TYPE_SALE_DECLINED = 'TxnSaleDeclined';

    private $client;

    public function __construct($registry) {
        parent::__construct($registry);
        $this->load->language('extension/payment/verifone_hosted');
        $this->load->model('checkout/order');
        $this->load->model('extension/payment/verifone_hosted');
        $this->model_extension_payment_verifone_hosted->__construct($registry);
        $this->client = $this->model_extension_payment_verifone_hosted->getClient();
    }

    public function process_webhook() {
        $this->load->language('extension/payment/verifone_hosted');
        $input = file_get_contents( 'php://input' );
        if ( ! $input ) {
            $this->api_response_error( 'Unable to process data' );
        }

        $data = json_decode( $input, true );
        if ( json_last_error() !== JSON_ERROR_NONE ) {
            $this->api_response_error( 'Unable to process data' );
        }

        if( $this->stringStartsWith( $data['eventType'], 'Checkout' ) ) {
            try {
                $checkout = $this->client->getCheckout( $data['recordId'] );
                $data['recordId'] = $checkout['transaction_id'];
            } catch ( Exception $exception ) {
                $this->api_response_error( 'Unable to process data' );
            }
        }

        try {
            $response = $this->client->getTransaction( $data['recordId'] );
        } catch ( Exception $exception ) {
            $this->api_response_error( 'Unable to process data' );
        }

        $order = null;
        if ( ! empty( $response['merchant_reference'] ) ) {
            $order = $this->model_checkout_order->getOrder( (int) $response['merchant_reference'] );
            if ( $order == false ) {
                $this->api_response_error( 'Unable to process data' );
            }

            $response['events'][] = ['type' => $response['status']];
            $response['transaction_id'] = $data['recordId'];
            $transaction = $response;
            $transaction['amount'] = (float)$transaction['amount'] * 100;
        }

        if ( $order == false ) {
            $this->api_response_error( 'Unable to process data' );
        }

        $this->logOrderComment( $data['eventType'], $order['order_id'], $order['order_status_id']);
        switch ( $response['status'] ) {
            case self::VERIFONE_TRANSACTION_AUTHORISED:
            case self::VERIFONE_TRANSACTION_SALE_AUTHORISED:
            case self::VERIFONE_TRANSACTION_SALE_SETTLED:
            case self::VERIFONE_TRANSACTION_AUTHORISATION_AUTHORISED:
            $this->model_extension_payment_verifone_hosted->addTransaction($transaction);
            $this->model_checkout_order->addOrderHistory(
                    $order['order_id'],
                    (int)$this->config->get('payment_verifone_hosted_order_status_id'),
                    ($this->language->get('entry_transaction_id') . '<strong style="color: #0D4AA2">' . $response['transaction_id'] . '</strong>')
                );
                break;

            case self::VERIFONE_TRANSACTION_PENDING:
                $this->model_extension_payment_verifone_hosted->addTransaction($transaction);
                $this->model_checkout_order->addOrderHistory(
                    $order['order_id'],
                    (int)$this->config->get('payment_verifone_hosted_pending_status_id'),
                    $this->language->get('text_payment_pending')
                );
                break;

            case self::VERIFONE_TRANSACTION_REFUND_AUTHORISED:
            case self::VERIFONE_TRANSACTION_REFUND_REFUNDED:
            case self::VERIFONE_TRANSACTION_REFUNDED:
            case self::VERIFONE_TRANSACTION_REFUND_SETTLED:
                $this->createRefundTransaction(self::VERIFONE_TRANSACTION_REFUNDED, $order['order_id'], $response['amount'], $this->language->get('text_order_payment_refunded'), $response['transaction_id']);
                $this->getRefundedAmounts($order['order_id']);
                if ($order['total'] == $this->getRefundedAmounts($order['order_id'])) {
                    $this->model_checkout_order->addOrderHistory(
                        $order['order_id'],
                        (int)$this->config->get('payment_verifone_hosted_refunded_status_id'),
                        $this->language->get('text_order_payment_refunded')
                    );
                } else {
                    $this->model_checkout_order->addOrderHistory(
                        $order['order_id'],
                        $order['order_status_id'],
                        $this->language->get('text_order_payment_refunded')
                    );
                }
                break;

            default:
                if ( self::VERIFONE_WEBHOOK_EVENT_TYPE_REFUND_DECLINED === $data['eventType'] ) {
                    $verifoneResponseAmount = (float) $response['amount'];

                    $refunds = $order->get_refunds();
                    foreach ( $refunds as $refund ) {
                        $refundAmount = (float) $refund->get_amount();
                        if ( 'completed' === $refund->get_status() && $verifoneResponseAmount === $refundAmount ) {
                            $refund->delete();
                            break;
                        }
                    }
                } else {
                    $this->model_checkout_order->addOrderHistory(
                        $order['order_id'],
                        (int)$this->config->get('payment_verifone_hosted_failed_status_id'),
                        $this->language->get('text_order_failed')
                    );
                }
        }

        $this->api_response_ok();
    }

    public function api_response( $message = null, $statusCode = 200 ) {
        http_response_code( $statusCode );
        echo $message;
        exit;
    }

    public function api_response_ok( $message = null ) {
        $this->api_response( $message, 200 );
    }

    public function api_response_error( $message = null ) {
        $this->api_response( $message, 500 );
    }

    private function logOrderComment( $status, $orderId, $orderStatusId) {
        $note = $this->language->get('text_notification') . $status;
        $this->model_checkout_order->addOrderHistory(
            $orderId,
            $orderStatusId,
            $note
        );
    }

    private function createRefundTransaction($status, $order_id, $amount, $comment, $transaction_id) {
        $q = $this->db->query("SELECT * FROM `" . DB_PREFIX . "verifone_hosted_refunds` WHERE `transaction_id` = '" . $transaction_id . "'");
        if(!$q->num_rows) {
            $this->db->query("
            INSERT INTO `" . DB_PREFIX . "verifone_hosted_refunds` SET
            `order_id` = '" . $order_id . "',
            `amount_refunded` = '" . $amount . "',
            `verifone_refund_status` = '" . $this->db->escape($status) . "',
            `comment` = '" . $this->db->escape($comment) . "',
            `response` = '" . $this->db->escape($this->language->get('entry_successfully_refunded')) . "',
            `transaction_id` = '" . $this->db->escape($transaction_id) . "'
            ");
        }
    }

    private function getRefundedAmounts($order_id) {
        $amount = 0;
        $q = $this->db->query("SELECT * FROM `" . DB_PREFIX . "verifone_hosted_refunds` WHERE `order_id` = '" . $order_id . "'");
        foreach ($q->rows as $row) {
            $amount += $row['amount_refunded'];
        }
        return $amount;
    }

    public function stringStartsWith($string, $needle) {
        $len = strlen($needle);
        return (substr($string, 0, $len) === $needle);
    }
}